; Barcode 128B
;~~~~~~~~~~~~~~

;{- Variablen

Structure Barcode_Data
   Code.s
   Value.a
   Char.a
EndStructure

NewList ct_Barcode.Barcode_Data()
Define.i ct_Barcode_PrintHeight, ct_Barcode_PrintZoom

;} EndVariablen

;{- Declarationen

Declare   ctUseBarcode()
Declare.s Barcode_Character(vBC.a)
Declare.s Barcode_Generate(vText.s)
Declare   ctBarcode_ScreenGenerate(vID.i, vText.s, vSpan.a = 1)
Declare   ctBarcode_PrintGenerate(vText.s, vX.i, vY.i, vHeight.i = 200, vZoom.a = 4)
Declare.i ctBarcode_PrintWidth(vText.s)
Declare.i ctBarcode_PrintHeight() 
Declare.i ctBarcode_PrintZoom()

;} EndDeclarationen

;Barcodeinitialisierung: Bei Programmanfang od. zur Standardeinstellung [Set]
Procedure ctUseBarcode()
   Shared ct_Barcode(), ct_Barcode_PrintHeight, ct_Barcode_PrintZoom
   Protected n.i, pCode.s
   
   If ListSize(ct_Barcode()): ClearList(ct_Barcode()): EndIf
   ct_Barcode_PrintHeight = 200
   ct_Barcode_PrintZoom   = 4
   Restore ctBarcode
   For n = 0 To 106
      Read.s pCode
      AddElement(ct_Barcode())
      With ct_Barcode()
         \Code = pCode
         \Value = n
         \Char  = n + 32
      EndWith
   Next n
EndProcedure

;Characterkonvertierung von ASCII in Barcode [Get|Intern]
Procedure.s Barcode_Character(vBC.a)
   Shared ct_Barcode()
   
   SelectElement(ct_Barcode(), vBC)
   ProcedureReturn ct_Barcode()\Code
EndProcedure

;Stringkonvertierung in Barcode [Get|Intern]
Procedure.s Barcode_Generate(vText.s)
;Barcodestring: <Start B> <Barcodetext> <Checksumme> <Stop>
   Shared ct_Barcode()
   Protected n.i, pChecksumme.i = 104, pValue.s = Barcode_Character(104)   
   
   For n=1 To Len(vText)
      ForEach ct_Barcode()
         With ct_Barcode()
            If (\Char=Asc(Mid(vText, n, 1)))
               pChecksumme + (\Value * n)
               pValue + Barcode_Character(\Value)
               Break
            EndIf
         EndWith
      Next
   Next
   pValue + Barcode_Character(pChecksumme%103)
   If (Len(pValue)%2): pValue + "0" : EndIf
   pValue + Barcode_Character(106)
   ProcedureReturn pValue
EndProcedure

;Barcodepresentation auf Canvasgadget [Set]
Procedure ctBarcode_ScreenGenerate(vID.i, vText.s,vZoom.a = 1)
   Protected n.i, pValue.s, pX.i, pY.i, pWidth.i, pHeight.i, pColor.i = #Black

   pValue + Barcode_Generate(vText)
   If StartDrawing(CanvasOutput(vID))
      Box( 0, 0, GadgetWidth(vID), GadgetHeight(vID), #White)
      pX = GadgetWidth(vID)
      For n=1 To Len(pValue): pX - (Val(Mid(pValue, n, 1)) * vZoom): Next n
      pX / 2: pY = 5 
      pHeight = (GadgetHeight(vID) - 15)
      For n=1 To Len(pValue)
         pWidth = Val(Mid(pValue, n, 1)) * vZoom
         Box(pX, pY, pWidth, pHeight, pColor)
         pX + pWidth
         If (pColor=#Black): pColor = #White: Else: pColor = #Black: EndIf
      Next n
      StopDrawing()
   EndIf
EndProcedure

;Barcodepresentation auf Drucker [Set]
Procedure ctBarcode_PrintGenerate(vText.s, vX.i, vY.i, vHeight.i = 200, vZoom.a = 4)
   Shared ct_Barcode_PrintHeight, ct_Barcode_PrintZoom
   Protected n.i, z.i, pValue.s, pColor.i = #Black
   
   ct_Barcode_PrintHeight = vHeight
   ct_Barcode_PrintZoom   = vZoom
   pValue + Barcode_Generate(vText)
   If StartDrawing(PrinterOutput())
      For n=1 To Len(pValue)
         For z=1 To (Val(Mid(pValue, n, 1)) * vZoom) 
            Line(vX, vY, 1, vHeight, pColor)
            vX + 1
         Next z
         If (pColor=#Black): pColor = #White: Else: pColor = #Black: EndIf
      Next n
      StopDrawing() 
   EndIf
EndProcedure

;Barcodebreite fr den Drucker [Get]
Procedure.i ctBarcode_PrintWidth(vText.s)
   Shared ct_Barcode_PrintZoom
   Protected n.i, pWidth.i
      
   vText = Barcode_Generate(vText)
   For n=1 To Len(vText)
      pWidth +  Val(Mid(vText, n, 1))
   Next n
   ProcedureReturn (pWidth * ct_Barcode_PrintZoom)
EndProcedure

;Barcodehhe fr den Drucker [Get]
Procedure.i ctBarcode_PrintHeight()
   Shared ct_Barcode_PrintHeight
   
   ProcedureReturn ct_Barcode_PrintHeight
EndProcedure

;Barcodegre fr den Drucker [Get]
Procedure.i ctBarcode_PrintZoom()
   Shared ct_Barcode_PrintZoom
   
   ProcedureReturn ct_Barcode_PrintZoom
EndProcedure

DataSection

   ctBarcode:   
      Data.s "212222","222122","222221","121223","121322","131222","122213"
      Data.s "122312","132212","221213","221312","231212","112232","122132"
      Data.s "122231","113222","123122","123221","223211","221132","221231"
      Data.s "213212","223112","312131","311222","321122","321221","312212"
      Data.s "322112","322211","212123","212321","232121","111323","131123"
      Data.s "131321","112313","132113","132311","211313","231113","231311"
      Data.s "112133","112331","132131","113123","113321","133121","313121"
      Data.s "211331","231131","213113","213311","213131","311123","311321"
      Data.s "331121","312113","312311","332111","314111","221411","431111"
      Data.s "111224","111422","121124","121421","141122","141221","112214"
      Data.s "112412","122114","122411","142112","142211","241211","221114"
      Data.s "413111","241112","134111","111242","121142","121241","114212"
      Data.s "124112","124211","411212","421112","421211","212141","214121"
      Data.s "412121","111143","111341","131141","114113","114311","411113"
      Data.s "411311","113141","114131","311141","411131"
      Data.s "211412","211214","211232","2331112"
EndDataSection
   